package com.sintecmedia.components.renderkit;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.component.UIParameter;
import javax.faces.context.FacesContext;

import org.ajax4jsf.javascript.JSFunction;
import org.ajax4jsf.javascript.JSFunctionDefinition;
import org.ajax4jsf.javascript.JSLiteral;
import org.ajax4jsf.renderkit.AjaxComponentRendererBase;
import org.ajax4jsf.renderkit.AjaxRendererUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.sintecmedia.components.component.UIWeekDaysButtons;
import com.sintecmedia.components.event.DaySelectedEvent;
import com.sintecmedia.components.exceptions.DayIndexOutOfBoundsException;

public abstract class WeekDaysButtonsRendererBase 
		extends AjaxComponentRendererBase {

	public static final String SELECTED_DAY_CLASS = "selected";
	public static final String UNSELECTED_DAY_CLASS = "unselected";
	
	private static final Log _log = 
		LogFactory.getLog( WeekDaysButtonsRendererBase.class );
	
	protected Class getComponentClass() {
		return UIWeekDaysButtons.class;
	}
	 		
	protected void doDecode( FacesContext context, UIComponent component ) {
		Map paramMap = getParamMap( context );
    	String clientId = component.getClientId( context );
    	
    	//If param is null, then there was no day selected in this request
    	//and no event should be queued.
    	String param = (String) paramMap.get( clientId );    	
        if( param == null ) {
        	return;
        }
                
		UIWeekDaysButtons dayButtons = (UIWeekDaysButtons) component;
		DaySelectedEvent event =
			new DaySelectedEvent( dayButtons, Integer.valueOf( param ) );
		event.queue();
	}

	private Map getParamMap(FacesContext context) {
        return context.getExternalContext().getRequestParameterMap();
    }

	/**
	 * @return JS expression what fires day selected event
	 */
	public String getOnClick( Integer dayIndex ) {
		return 	"changeSelection(this," + dayIndex + ")";	
	}

	/**
	 * @param context current JSF context
	 * @param component component index of week day requested for 
	 * @param dayName name of day from day of week day labels of the component
	 * @return returns index of day with name specified if this name is one of week day labels of the component
	 * @throws DayIndexOutOfBoundsException if day name is not one of week day labels of the component
	 */
	public Integer getIndexByDay( FacesContext context, UIComponent component, Object dayName ) 
			throws DayIndexOutOfBoundsException {
		try {
		return		
			Calendar.getInstance().getDisplayNames(
					Calendar.DAY_OF_WEEK, Calendar.LONG, Locale.ENGLISH 
				).get( String.valueOf( dayName ) ) -1 ;
		} catch( Exception e ) {
			DayIndexOutOfBoundsException ex =  
				new DayIndexOutOfBoundsException();
			ex.setComponent( component );
			ex.setContext( context );
			ex.setMethodName( "getIndexByDay" );
			ex.setStackTrace( ex.getStackTrace() );
			throw ex;
		}
	}
	
	/**
	 * 
	 * @param context
	 * @param component
	 * @return code of Javascript function that submits event through AJAX request
	 */
	public String getSubmitFunction(
			FacesContext context, UIComponent component ) {
		JSFunctionDefinition definition = new JSFunctionDefinition("event");
	        
		JSFunction function = 
			AjaxRendererUtils.buildAjaxFunction( component, context );
        Map eventOptions = 
        	AjaxRendererUtils.buildEventOptions( context, component, true );
        Map parameters = (Map) eventOptions.get( "parameters" );
        
        Map params = getParameters( context,component );
        if(!params.isEmpty()){
        	parameters.putAll(params);
        } 
        
        parameters.put(
        		component.getClientId(context), 
        		new JSLiteral("event.memo.dayIndex") );        
        function.addParameter(eventOptions);
        StringBuffer buffer = new StringBuffer();
        function.appendScript(buffer);
        buffer.append("; return false;");
		definition.addToBody(buffer.toString());			        
		return definition.toScript();
	}
	
	//get UIParameter's Map
    protected Map getParameters( FacesContext context, UIComponent component ) {
    	Map parameters = new HashMap(); 
    	
    	if( component instanceof UIWeekDaysButtons ){
    		UIWeekDaysButtons weekDaysButtons = (UIWeekDaysButtons)component;
    		List children = weekDaysButtons.getChildren();
    		for (Iterator iterator = children.iterator(); iterator.hasNext();) {
				UIComponent child = (UIComponent) iterator.next();
				if(child instanceof UIParameter) {
					UIParameter param = (UIParameter)child;
					String name = param.getName();
					if (name != null) {
						parameters.put(name, param.getValue());
					}
				}
			}
    	}
    	    	
    	return parameters;
    }
    
	/**
	 * 
	 * @param component UIWeeklyCalendar component used.
	 * @return list of days from component 
	 */
	public List<String> getDaysOfWeek( FacesContext context, UIComponent component ) {
		//Days map returned from calendar relays on day's name as key, so it
		//needs refinement by day's index.
		Map<String, Integer> daysMap = 
			Calendar.getInstance().getDisplayNames(
					Calendar.DAY_OF_WEEK, Calendar.LONG, Locale.ENGLISH );
		String[] names = new String[ daysMap.size() ]; 
		for( String name : daysMap.keySet() ) {
			names[ daysMap.get( name ) - 1 ] = name;
		}			
		
		List<String> result = new ArrayList<String>( 7 );
		for( int i = 0; i < 7; i++ ) {			
			result.add( names[i] );
		}
		return result;
	}
	
	public String getClassForDay( UIComponent component, Integer dayIndex ) {
		return 
			( (UIWeekDaysButtons)component ).isDaySelected( dayIndex ) ? 
					SELECTED_DAY_CLASS : UNSELECTED_DAY_CLASS;	
	}
}
